#include "ep_lib.hpp"
#include <mpi.h>
#include "ep_declaration.hpp"
#include "ep_mpi.hpp"

using namespace std;

namespace ep_lib
{

	int MPI_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, 
                     int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
  {
    int target_mpi_rank   = win.comm.rank_map->at(target_rank).second;
    int target_local_rank = win.comm.rank_map->at(target_rank).first;
    int num_ep = win.comm.ep_comm_ptr->size_rank_info[1].second;
    if(num_ep==1)
      return  ::MPI_Accumulate(origin_addr, origin_count, to_mpi_type(origin_datatype), target_mpi_rank, to_mpi_aint(target_disp), target_count, to_mpi_type(target_datatype), 
                               to_mpi_op(op), to_mpi_win(win.server_win[target_local_rank]));

    else
    	return  ::MPI_Accumulate(origin_addr, origin_count, to_mpi_type(origin_datatype), target_mpi_rank, to_mpi_aint(target_disp), target_count, to_mpi_type(target_datatype), 
                               to_mpi_op(op), to_mpi_win(win.client_win));
  }

  int MPI_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, 
                         int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp,
                         int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win)
  {
    int target_mpi_rank   = win.comm.rank_map->at(target_rank).second;
    int target_local_rank = win.comm.rank_map->at(target_rank).first;
    int num_ep = win.comm.ep_comm_ptr->size_rank_info[1].second;
    if(num_ep==1)
      return ::MPI_Get_accumulate(origin_addr, origin_count, to_mpi_type(origin_datatype), result_addr, result_count, to_mpi_type(result_datatype), target_mpi_rank, to_mpi_aint(target_disp), target_count,
                                  to_mpi_type(target_datatype), to_mpi_op(op), to_mpi_win(win.server_win[target_local_rank]));
    else
      return ::MPI_Get_accumulate(origin_addr, origin_count, to_mpi_type(origin_datatype), result_addr, result_count, to_mpi_type(result_datatype), target_mpi_rank, to_mpi_aint(target_disp), target_count,
                                  to_mpi_type(target_datatype), to_mpi_op(op), to_mpi_win(win.client_win));
  }

}